@@ -31,8 +31,7 @@ G_DECLARE_FINAL_TYPE (GtdProviderTodoTxt, gtd_provider_todo_txt, GTD, PROVIDER_T
 
 GtdProviderTodoTxt*    gtd_provider_todo_txt_new                     (GFile         *source_file);
 
-void                   gtd_provider_todo_txt_set_monitor             (GtdProviderTodoTxt *self,
-                                                                      GFileMonitor       *monitor);
+GtdTask* create_task (void);
 
 G_END_DECLS
 
