@@ -37,8 +37,7 @@ G_DECLARE_FINAL_TYPE (GtdTaskListView, gtd_task_list_view, GTD, TASK_LIST_VIEW,
  * @before_task: the #GtdTask that @before represents
  * @user_data: (closure): user data
  *
- * Will be called when the primary or secondary action of @notification
- * is executed.
+ * The header function called on every task.
  */
 typedef void (*GtdTaskListViewHeaderFunc) (GtkListBoxRow        *row,
                                            GtdTask              *row_task,
@@ -46,6 +45,22 @@ typedef void (*GtdTaskListViewHeaderFunc) (GtkListBoxRow        *row,
                                            GtdTask              *before_task,
                                            gpointer              user_data);
 
+/**
+ * GtdTaskListViewSortFunc:
+ * @row1: the current #GtkListBoxRow
+ * @row1_task: the #GtdTask that @row represents
+ * @row2: the #GtkListBoxRow before @row
+ * @row2_task: the #GtdTask that @before represents
+ * @user_data: (closure): user data
+ *
+ * The sorting function called on every task.
+ */
+typedef gint (*GtdTaskListViewSortFunc)   (GtkListBoxRow        *row1,
+                                           GtdTask              *row1_task,
+                                           GtkListBoxRow        *row2,
+                                           GtdTask              *row2_task,
+                                           gpointer              user_data);
+
 GtkWidget*                gtd_task_list_view_new                (void);
 
 GList*                    gtd_task_list_view_get_list           (GtdTaskListView        *view);
@@ -53,11 +68,6 @@ GList*                    gtd_task_list_view_get_list           (GtdTaskListView
 void                      gtd_task_list_view_set_list           (GtdTaskListView        *view,
                                                                  GList                  *list);
 
-gboolean                  gtd_task_list_view_get_readonly       (GtdTaskListView        *view);
-
-void                      gtd_task_list_view_set_readonly       (GtdTaskListView        *view,
-                                                                 gboolean                readonly);
-
 GtdTaskList*              gtd_task_list_view_get_task_list      (GtdTaskListView        *view);
 
 void                      gtd_task_list_view_set_task_list      (GtdTaskListView        *view,
@@ -68,6 +78,11 @@ gboolean                  gtd_task_list_view_get_show_list_name (GtdTaskListView
 void                      gtd_task_list_view_set_show_list_name (GtdTaskListView        *view,
                                                                  gboolean                show_list_name);
 
+gboolean                  gtd_task_list_view_get_show_due_date  (GtdTaskListView        *self);
+
+void                      gtd_task_list_view_set_show_due_date  (GtdTaskListView        *self,
+                                                                 gboolean                show_due_date);
+
 gboolean                  gtd_task_list_view_get_show_completed (GtdTaskListView        *view);
 
 void                      gtd_task_list_view_set_show_completed (GtdTaskListView        *view,
@@ -77,6 +92,30 @@ void                      gtd_task_list_view_set_header_func    (GtdTaskListView
                                                                  GtdTaskListViewHeaderFunc  func,
                                                                  gpointer                   user_data);
 
+void                      gtd_task_list_view_set_sort_func      (GtdTaskListView           *view,
+                                                                 GtdTaskListViewSortFunc    func,
+                                                                 gpointer                   user_data);
+
+gboolean                  gtd_task_list_view_get_show_new_task_row (GtdTaskListView        *view);
+
+void                      gtd_task_list_view_set_show_new_task_row (GtdTaskListView        *view,
+                                                                    gboolean                show_new_task_row);
+
+GDateTime*                gtd_task_list_view_get_default_date   (GtdTaskListView        *self);
+
+void                      gtd_task_list_view_set_default_date   (GtdTaskListView        *self,
+                                                                 GDateTime              *default_date);
+
+GdkRGBA*                  gtd_task_list_view_get_color          (GtdTaskListView        *self);
+
+void                      gtd_task_list_view_set_color          (GtdTaskListView        *self,
+                                                                 GdkRGBA                *color);
+
+gboolean                  gtd_task_list_view_get_handle_subtasks (GtdTaskListView       *self);
+
+void                      gtd_task_list_view_set_handle_subtasks (GtdTaskListView       *self,
+                                                                  gboolean               handle_subtasks);
+
 G_END_DECLS
 
 #endif /* GTD_TASK_LIST_VIEW_H */
