@@ -23,12 +23,32 @@
 #include <glib/gi18n.h>
 #include <libecal/libecal.h>
 
+/**
+ * SECTION:gtd-task-list
+ * @short_description:a list of tasks
+ * @title:GtdTaskList
+ * @stability:Unstable
+ * @see_also:#GtdTask
+ *
+ * A #GtdTaskList represents a task list, and contains a list of tasks, a color,
+ * a name and the provider who generated it.
+ */
+
+typedef struct
+{
+  gchar   *parent_uid;
+  GtdTask *child;
+} PendingSubtaskData;
+
 typedef struct
 {
   GList               *tasks;
   GtdProvider         *provider;
   GdkRGBA             *color;
 
+  GHashTable          *uid_to_task;
+  GPtrArray           *pending_subtasks;
+
   gchar               *name;
   gboolean             removable : 1;
 } GtdTaskListPrivate;
@@ -55,13 +75,109 @@ enum
   LAST_PROP
 };
 
+static PendingSubtaskData*
+pending_subtask_data_new (GtdTaskList *self,
+                          GtdTask     *child,
+                          const gchar *parent_uid)
+{
+  PendingSubtaskData *data;
+
+  data = g_new0 (PendingSubtaskData, 1);
+  data->child = child;
+  data->parent_uid = g_strdup (parent_uid);
+
+  return data;
+}
+
+static void
+pending_subtask_data_free (PendingSubtaskData *data)
+{
+  g_free (data->parent_uid);
+  g_free (data);
+}
+
+static void
+setup_parent_task (GtdTaskList *self,
+                   GtdTask     *task)
+{
+  GtdTaskListPrivate *priv;
+  ECalComponent *component;
+  icalcomponent *ical_comp;
+  icalproperty *property;
+  GtdTask *parent_task;
+  const gchar *parent_uid;
+
+  priv = gtd_task_list_get_instance_private (self);
+  component = gtd_task_get_component (task);
+  ical_comp = e_cal_component_get_icalcomponent (component);
+  property = icalcomponent_get_first_property (ical_comp, ICAL_RELATEDTO_PROPERTY);
+
+  if (!property)
+    return;
+
+  parent_uid = icalproperty_get_relatedto (property);
+  parent_task = g_hash_table_lookup (priv->uid_to_task, parent_uid);
+
+  if (parent_task)
+    {
+      gtd_task_add_subtask (parent_task, task);
+    }
+  else
+    {
+      PendingSubtaskData *data;
+
+      data = pending_subtask_data_new (self, task, parent_uid);
+
+      g_ptr_array_add (priv->pending_subtasks, data);
+    }
+}
+
+static void
+process_pending_subtasks (GtdTaskList *self,
+                          GtdTask     *task)
+{
+  GtdTaskListPrivate *priv;
+  ECalComponentId *id;
+  guint i;
+
+  priv = gtd_task_list_get_instance_private (self);
+  id = e_cal_component_get_id (gtd_task_get_component (task));
+
+  for (i = 0; i < priv->pending_subtasks->len; i++)
+    {
+      PendingSubtaskData *data;
+
+      data = g_ptr_array_index (priv->pending_subtasks, i);
+
+      if (g_strcmp0 (id->uid, data->parent_uid) == 0)
+        {
+          gtd_task_add_subtask (task, data->child);
+          g_ptr_array_remove (priv->pending_subtasks, data);
+          i--;
+        }
+    }
+
+  e_cal_component_free_id (id);
+}
+
+static void
+task_changed_cb (GtdTask     *task,
+                 GParamSpec  *pspec,
+                 GtdTaskList *self)
+{
+  g_signal_emit (self, signals[TASK_UPDATED], 0, task);
+}
+
 static void
 gtd_task_list_finalize (GObject *object)
 {
   GtdTaskList *self = (GtdTaskList*) object;
   GtdTaskListPrivate *priv = gtd_task_list_get_instance_private (self);
 
+  g_clear_object (&priv->pending_subtasks);
   g_clear_object (&priv->provider);
+
+  g_clear_pointer (&priv->uid_to_task, g_hash_table_destroy);
   g_clear_pointer (&priv->color, gdk_rgba_free);
   g_clear_pointer (&priv->name, g_free);
 
@@ -80,8 +196,12 @@ gtd_task_list_get_property (GObject    *object,
   switch (prop_id)
     {
     case PROP_COLOR:
-      g_value_set_boxed (value, gtd_task_list_get_color (self));
-      break;
+      {
+        GdkRGBA *color = gtd_task_list_get_color (self);
+        g_value_set_boxed (value, color);
+        gdk_rgba_free (color);
+        break;
+      }
 
     case PROP_IS_REMOVABLE:
       g_value_set_boolean (value, gtd_task_list_is_removable (self));
@@ -107,7 +227,6 @@ gtd_task_list_set_property (GObject      *object,
                             GParamSpec   *pspec)
 {
   GtdTaskList *self = GTD_TASK_LIST (object);
-  GtdTaskListPrivate *priv = gtd_task_list_get_instance_private (self);
 
   switch (prop_id)
     {
@@ -124,8 +243,7 @@ gtd_task_list_set_property (GObject      *object,
       break;
 
     case PROP_PROVIDER:
-      if (g_set_object (&priv->provider, g_value_get_object (value)))
-        g_object_notify (object, "provider");
+      gtd_task_list_set_provider (self, g_value_get_object (value));
       break;
 
     default:
@@ -196,7 +314,7 @@ gtd_task_list_class_init (GtdTaskListClass *klass)
                              "Provider of the list",
                              "The provider that handles the list",
                              GTD_TYPE_PROVIDER,
-                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                             G_PARAM_READWRITE));
 
   /**
    * GtdTaskList::task-added:
@@ -259,7 +377,16 @@ gtd_task_list_class_init (GtdTaskListClass *klass)
 static void
 gtd_task_list_init (GtdTaskList *self)
 {
-  ;
+  GtdTaskListPrivate *priv;
+
+  priv = gtd_task_list_get_instance_private (self);
+
+  priv->uid_to_task = g_hash_table_new_full (g_str_hash,
+                                             g_str_equal,
+                                             g_free,
+                                             NULL);
+
+  priv->pending_subtasks = g_ptr_array_new_with_free_func ((GDestroyNotify) pending_subtask_data_free);
 }
 
 /**
@@ -376,6 +503,7 @@ gtd_task_list_set_name (GtdTaskList *list,
 
   if (g_strcmp0 (priv->name, name) != 0)
     {
+      g_free (priv->name);
       priv->name = g_strdup (name);
 
       g_object_notify (G_OBJECT (list), "name");
@@ -402,6 +530,27 @@ gtd_task_list_get_provider (GtdTaskList *list)
   return priv->provider;
 }
 
+/**
+ * gtd_task_list_set_provider:
+ * @self: a #GtdTaskList
+ * @provider: (nullable): a #GtdProvider, or %NULL
+ *
+ * Sets the provider of this tasklist.
+ */
+void
+gtd_task_list_set_provider (GtdTaskList *self,
+                            GtdProvider *provider)
+{
+  GtdTaskListPrivate *priv;
+
+  g_assert (GTD_IS_TASK_LIST (self));
+
+  priv = gtd_task_list_get_instance_private (self);
+
+  if (g_set_object (&priv->provider, provider))
+    g_object_notify (G_OBJECT (self), "provider");
+}
+
 /**
  * gtd_task_list_get_tasks:
  * @list: a #GtdTaskList
@@ -446,9 +595,24 @@ gtd_task_list_save_task (GtdTaskList *list,
     }
   else
     {
+      ECalComponentId *id;
+
+      id = e_cal_component_get_id (gtd_task_get_component (task));
+
       priv->tasks = g_list_append (priv->tasks, task);
 
+      g_hash_table_insert (priv->uid_to_task, g_strdup (id->uid), task);
+      process_pending_subtasks (list, task);
+      setup_parent_task (list, task);
+
+      g_signal_connect (task,
+                        "notify",
+                        G_CALLBACK (task_changed_cb),
+                        list);
+
       g_signal_emit (list, signals[TASK_ADDED], 0, task);
+
+      e_cal_component_free_id (id);
     }
 }
 
@@ -473,8 +637,14 @@ gtd_task_list_remove_task (GtdTaskList *list,
   if (!gtd_task_list_contains (list, task))
     return;
 
+  g_signal_handlers_disconnect_by_func (task,
+                                        task_changed_cb,
+                                        list);
+
   priv->tasks = g_list_remove (priv->tasks, task);
 
+  g_hash_table_remove (priv->uid_to_task, gtd_object_get_uid (GTD_OBJECT (task)));
+
   g_signal_emit (list, signals[TASK_REMOVED], 0, task);
 }
 
@@ -498,7 +668,7 @@ gtd_task_list_contains (GtdTaskList *list,
 
   priv = gtd_task_list_get_instance_private (list);
 
-  return (g_list_find (priv->tasks, task) != NULL);
+  return g_list_find (priv->tasks, task) != NULL;
 }
 
 /**
